@@ -1639,10 +1639,14 @@ static int readline_select_file(int count, int key)
                             , cursor_path+strlen(cwd_path)
                             , strlen(cursor_path) - strlen(cwd_path)+1);
 
-                        rl_insert_text(cursor_path2);
+                        string_obj* str = STRING_NEW("");
+                        saphire_get_quoted_fname(cursor_path2, str);
+                        rl_insert_text(string_c_str(str));
                     }
                     else {
-                        rl_insert_text(cursor_path);
+                        string_obj* str = STRING_NEW("");
+                        saphire_get_quoted_fname(cursor_path, str);
+                        rl_insert_text(string_c_str(str));
                     }
                 }
                 else {
@@ -1660,13 +1664,17 @@ static int readline_select_file(int count, int key)
                                 , strlen(full_path) - strlen(cwd_path));
                             full_path2[strlen(full_path) - strlen(cwd_path)] = 0;
 
-                            rl_insert_text(full_path2);
+                            string_obj* str = STRING_NEW("");
+                            saphire_get_quoted_fname(full_path2, str);
+                            rl_insert_text(string_c_str(str));
 
                             if(j<vector_size(mark_files)-1) 
                                 rl_insert_text(" ");
                         }
                         else {
-                            rl_insert_text(full_path);
+                            string_obj* str = STRING_NEW("");
+                            saphire_get_quoted_fname(full_path, str);
+                            rl_insert_text(string_c_str(str));
 
                             if(j<vector_size(mark_files)-1) 
                                 rl_insert_text(" ");
