@@ -27,18 +27,9 @@ typedef struct {
 // ֐錾
 ///////////////////////////////////////////////////////////////////////
 // 
-#if defined(MDEBUG)
-
-list_obj* list_new_debug(const char* fname, int line, const char* func_name);
-#define LIST_NEW() list_new_debug(__FILE__, __LINE__, __FUNCTION__)
-
-#else
-
 list_obj* list_new();
 #define LIST_NEW() list_new()
 
-#endif
-
 void list_delete(list_obj* self);       // 
 
 int list_size(list_obj* self);                      // TCY
