@@ -36,7 +36,7 @@ enum eKanjiCode gKanjiCode = kUtf8;
 
 
 char* gStatmentKindStrs[kCommand+1] = {
-    "object", "pointer", "var", "global", "export", 
+    "object", "ref", "var", "global", "export", 
     "ary", "ary_add", "ary_erase", 
     "hash", "hash_add", "hash_erase",
     "def", "inherit", "self", "class",
@@ -116,13 +116,12 @@ hash_obj* gHashs;
     // グローバル変数の配列 キーは変数名
     // 内容はハッシュ(hash_obj*) ハッシュの中にはstring_obj*
     // が入っている
-hash_obj* gPointers;
-    // ポインター キーは変数名
 
 vector_obj* gStackFrame;
-vector_obj* gStackFramePointers;
 vector_obj* gStackFrameArrays;
 vector_obj* gStackFrameHashs;
+vector_obj* gStackFrameObjects;
+vector_obj* gStackFrameRefs;
     // ローカル変数のスタック
     // 内容はhash_obj*
     // これのキーはローカル変数名
@@ -190,9 +189,12 @@ BOOL gSaphireBackground;
 hash_obj* gObjects;
     // オブジェクト キーはオブジェクト名 内容はsObject*
 
+hash_obj* gRefs;
+    // リファレンス
+
 sObject* sObject_new(char* name, sClass* klass, sObject* parent) 
 {
-    sObject* self = MALLOC(sizeof(sObject));
+    sObject* self = memchecker_GC_malloc(sizeof(sObject), kMCObject);
 
     self->mName = STRING_NEW(name);
     self->mClass = klass;
@@ -200,8 +202,8 @@ sObject* sObject_new(char* name, sClass* klass, sObject* parent)
     self->mHashs = HASH_NEW(15);
     self->mArrays = HASH_NEW(15);
     self->mObjects = HASH_NEW(15);
+    self->mRefs = HASH_NEW(15);
     self->mClasses = HASH_NEW(15);
-    self->mPointers = HASH_NEW(15);
 
     self->mParent = parent;
 
@@ -404,7 +406,7 @@ void saphire_set_local_var(char* name, char* value)
         string_put(var, value);
     }
     else {
-        hash_put(top_stack , name, STRING_NEW(value));
+        hash_put(top_stack , name, sVar_new(value));
     }
 }
 
@@ -412,12 +414,14 @@ void saphire_init_stack_frame()
 {
     vector_clear(gStackFrame);
     vector_add(gStackFrame, HASH_NEW(30));
-    vector_clear(gStackFramePointers);
-    vector_add(gStackFramePointers, HASH_NEW(30));
     vector_clear(gStackFrameArrays);
     vector_add(gStackFrameArrays, HASH_NEW(30));
     vector_clear(gStackFrameHashs);
     vector_add(gStackFrameHashs, HASH_NEW(30));
+    vector_clear(gStackFrameObjects);
+    vector_add(gStackFrameObjects, HASH_NEW(30));
+    vector_clear(gStackFrameRefs);
+    vector_add(gStackFrameRefs, HASH_NEW(30));
 }
 
 void saphire_set_global_var(char* name, char* value)
@@ -425,9 +429,11 @@ void saphire_set_global_var(char* name, char* value)
     string_obj* global = hash_item(gGlobals, name);
 
     if(global) {
-        hash_erase(gGlobals, name);
+        string_put(global, value);
+    }
+    else {
+        hash_put(gGlobals, name, sGlobal_new(value));
     }
-    hash_put(gGlobals, name, STRING_NEW(value));
 }
 
 void saphire_set_global_var2(char* name, char* value, sObject* object)
@@ -435,9 +441,11 @@ void saphire_set_global_var2(char* name, char* value, sObject* object)
     string_obj* global = hash_item(object->mGlobals, name);
 
     if(global) {
-        hash_erase(object->mGlobals, name);
+        string_put(global, value);
+    }
+    else {
+        hash_put(object->mGlobals, name, sGlobal_new(value));
     }
-    hash_put(object->mGlobals, name, STRING_NEW(value));
 }
 
 
@@ -489,27 +497,6 @@ char* saphire_get_local_var(char* name)
     }
 }
 
-void saphire_delete_local_var(char* name)
-{
-    hash_obj* top_stack 
-        = vector_item(gStackFrame, vector_size(gStackFrame)-1);
-
-    string_obj* var = hash_item(top_stack, name);
-
-    if(var) {
-        hash_erase(top_stack, name);
-    }
-
-    char name_size[1024];
-    snprintf(name_size, 1024, "%s_size", name);
-
-    string_obj* var_size = hash_item(gGlobals, name_size);
-
-    if(var_size) {
-        hash_erase(gGlobals, name_size);
-    }
-}
-
 //////////////////////////////////////////////////////////
 // シグナル処理
 //////////////////////////////////////////////////////////
@@ -1170,51 +1157,6 @@ void sSubshell_view(sSubshell* self)
     sStatments_view(self->contents);
 }
 
-//////////////////////////////////////////////////////////
-// sTry
-//////////////////////////////////////////////////////////
-sTry* sTry_new(MANAGED sStatments* loops, MANAGED sStatments* loops2)
-{
-    sTry* self = MALLOC(sizeof(sTry));
-
-    self->loops = loops;
-    self->loops2 = loops2;
-
-    return self;
-}
-
-sTry* sTry_new2(sTry* try_)
-{
-    sTry* self = MALLOC(sizeof(sTry));
-
-    self->loops = sStatments_new2(try_->loops);
-    self->loops2 = sStatments_new2(try_->loops2);
-
-    return self;
-}
-
-void sTry_view(sTry* self)
-{
-    sStatments_view(self->loops);
-    sStatments_view(self->loops2);
-}
-
-void sTry_save(sTry* self, int fd)
-{
-    sStatments_save(self->loops, fd);
-    sStatments_save(self->loops2, fd);
-}
-
-sTry* sTry_load(int fd)
-{
-    sTry* self = MALLOC(sizeof(sTry));
-    
-    self->loops = sStatments_load(fd);
-    self->loops2 = sStatments_load(fd);
-    
-    return self;
-}
-
 //////////////////////////////////////////////////////////
 // sWhile
 //////////////////////////////////////////////////////////
@@ -1978,7 +1920,7 @@ sStatments* sStatments_load(int fd)
 /////////////////////////////////////////////////////////
 sClass* sClass_new(char* name, sStatments* statments, sClass* parent)
 {
-    sClass* self = MALLOC(sizeof(sClass));
+    sClass* self = memchecker_GC_malloc(sizeof(sClass), kMCClass);
     self->name = STRING_NEW(name);
     self->statments = statments;
     self->parent = parent;
@@ -1989,7 +1931,7 @@ sClass* sClass_new(char* name, sStatments* statments, sClass* parent)
 
 sClass* sClass_new2(sClass* klass)
 {
-    sClass* self = MALLOC(sizeof(sClass));
+    sClass* self = memchecker_GC_malloc(sizeof(sClass), kMCClass);
     self->name = STRING_NEW(string_c_str(klass->name));
     
     if(klass->statments) {
@@ -2045,7 +1987,7 @@ void sClass_save(sClass* self, int fd)
 
 sClass* sClass_load(int fd)
 {
-    sClass* self = MALLOC(sizeof(sClass));
+    sClass* self = memchecker_GC_malloc(sizeof(sClass), kMCClass);
     
     int n;
     if(read(fd, &n, sizeof(int)) < 0) {
@@ -2087,7 +2029,7 @@ void sClass_view(sClass* self)
 /////////////////////////////////////////////////////////
 sFunction* sFunction_new(char* name, sStatments* statments, char* arg_name, sFunction* parent)
 {
-    sFunction* self = MALLOC(sizeof(sFunction));
+    sFunction* self = memchecker_GC_malloc(sizeof(sFunction), kMCFunction);
 
     self->name = STRING_NEW(name);
     if(arg_name) {
@@ -2106,7 +2048,7 @@ sFunction* sFunction_new(char* name, sStatments* statments, char* arg_name, sFun
 
 sFunction* sFunction_new2(sFunction* func)
 {
-    sFunction* self = MALLOC(sizeof(sFunction));
+    sFunction* self = memchecker_GC_malloc(sizeof(sFunction), kMCFunction);
     self->name = STRING_NEW(string_c_str(func->name));
     if(func->arg_name) {
         self->arg_name = STRING_NEW(string_c_str(func->arg_name));
@@ -2190,7 +2132,7 @@ void sFunction_save(sFunction* self, int fd)
 
 sFunction* sFunction_load(int fd)
 {
-    sFunction* self = MALLOC(sizeof(sFunction));
+    sFunction* self = memchecker_GC_malloc(sizeof(sFunction), kMCFunction);
     
     int n;
     if(read(fd, &n, sizeof(int)) < 0) {
@@ -2570,6 +2512,8 @@ static void read_rc_obj_file(BOOL run_user_runtimescript)
 ////////////////////////////////////////////////////////
 void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript runtime_script, BOOL run_user_runtimescript)
 {
+    memchecker_init();
+
     gGlobalPipe = STRING_NEW("");
     int i;
     for(i=0; i<10; i++) {
@@ -2585,7 +2529,6 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     gHashs = HASH_NEW(100);
     gFuncs = HASH_NEW(50);
     gClasses = HASH_NEW(50);
-    gPointers = HASH_NEW(50);
 
     gJobs = VECTOR_NEW(50);
 
@@ -2593,9 +2536,11 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
 
     gObjects = HASH_NEW(50);
 
+    gRefs = HASH_NEW(50);
+
     gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "3.4.0", 1);
+    setenv("SAPHIRE_VERSION", "3.4.3", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
@@ -2624,10 +2569,12 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     vector_add(gStackFrame, HASH_NEW(30));
     gStackFrameArrays = VECTOR_NEW(5);
     vector_add(gStackFrameArrays, HASH_NEW(30));
-    gStackFramePointers = VECTOR_NEW(5);
-    vector_add(gStackFramePointers, HASH_NEW(30));
     gStackFrameHashs = VECTOR_NEW(5);
     vector_add(gStackFrameHashs, HASH_NEW(30));
+    gStackFrameObjects = VECTOR_NEW(5);
+    vector_add(gStackFrameObjects, HASH_NEW(30));
+    gStackFrameRefs = VECTOR_NEW(5);
+    vector_add(gStackFrameRefs, HASH_NEW(30));
 
     gInnerCommands = HASH_NEW(50);
     gKitutukiExit = -1;
@@ -2787,7 +2734,7 @@ int sRFd_read_all(sRFd* self, string_obj* str)
                 return -1;
             }
             if(r == 0) {
-                if(string_length(str) == 0) {
+                if(strlen(self->mBuffer) == 0) {
                     return 1;
                 }
                 break;
@@ -2837,7 +2784,7 @@ int sRFd_read_all_preserve(sRFd* self, string_obj* str)
                 return -1;
             }
             if(r == 0) {
-                if(self->mPtr == self->mBuffer + strlen(self->mBuffer)) {
+                if(strlen(self->mBuffer) == 0) {
                     return 1;
                 }
                 break;
@@ -3854,18 +3801,6 @@ void saphire_make_var_completion_list()
         it = hash_loop_next(it);
     }
 
-    it = hash_loop_begin(gPointers);
-    while(it) {
-        char* key = hash_loop_key(it);
-
-        char env_name[1024];
-        xstrncpy(env_name, key, 1024);
-
-        vector_add(gSaphireVarCompletions, STRING_NEW(env_name));
-
-        it = hash_loop_next(it);
-    }
-
     (void)vector_sort(gSaphireVarCompletions, name_sort);
 }
 
@@ -3995,21 +3930,106 @@ sObject* saphire_get_object(char* name)
     return hash_item(gObjects, name);
 }
 
-sPointer* sPointer_new(char* name, string_obj* pointer)
+
+string_obj* sVar_new(char* str)
 {
-    sPointer* self = MALLOC(sizeof(sPointer));
+    const int len = strlen(str);
     
-    self->mName = STRING_NEW(name);
-    self->mPointer = pointer;
-    self->mP = string_c_str(pointer);
+    string_obj* self = memchecker_GC_malloc(sizeof(string_obj), kMCVar);
+    
+    self->mMallocLen = (len < 128) ? 128 : (len + 1) * 2;
+    self->mStr = (char*)MALLOC(sizeof(char)*self->mMallocLen);
+    xstrncpy(self->mStr, str, self->mMallocLen);
+    
+    self->mLen = len;
 
     return self;
 }
 
-BOOL sPointer_inc(sPointer* self, int n, enum eKanjiCode code)
+string_obj* sGlobal_new(char* str)
 {
-    self->mP = str_kanjipos2pointer(code, self->mP, n);
+    const int len = strlen(str);
+    
+    string_obj* self = memchecker_GC_malloc(sizeof(string_obj), kMCGlobal);
+    
+    self->mMallocLen = (len < 128) ? 128 : (len + 1) * 2;
+    self->mStr = (char*)MALLOC(sizeof(char)*self->mMallocLen);
+    xstrncpy(self->mStr, str, self->mMallocLen);
+    
+    self->mLen = len;
 
-    return (*self->mP) != 0;
+    return self;
 }
 
+vector_obj* sAry_new(int first_size)
+{
+   vector_obj* self = (vector_obj*)memchecker_GC_malloc(sizeof(vector_obj), kMCArray);
+
+   self->mTable = (void**)MALLOC(sizeof(void*) * first_size);
+   self->mTableSize = first_size;
+   self->mCount = 0;
+
+   return self;
+}
+
+hash_obj* sHash_new(int size)
+{
+   hash_obj* self = (hash_obj*)memchecker_GC_malloc(sizeof(hash_obj), kMCHash);
+   
+   self->mTableSize = size;
+   self->mTable = (hash_it**)MALLOC(sizeof(hash_it*) * size);
+   memset(self->mTable, 0, sizeof(hash_it*)*size);
+
+   self->mEntryIt = NULL;
+
+   self->mCounter = 0;
+
+   return self;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+// メモリチェッカー
+///////////////////////////////////////////////////////////////////////////////
+static hash_obj* gMemChecker;
+
+void memchecker_init()
+{
+    gMemChecker = HASH_NEW(8096);
+}
+
+void memchecker_finalizer(void* obj, void* client_data)
+{
+    char buf[512];
+    snprintf(buf, 512, "%d", (unsigned int)obj);
+    hash_erase(gMemChecker, buf);
+}
+
+// オブジェクトの種類を返す 0:無し 1:
+int memchecker_is_enable_mem(void* obj)
+{
+    char buf[512];
+    snprintf(buf, 512, "%d", (unsigned int)obj);
+    return (int)hash_item(gMemChecker, buf);
+}
+
+void* memchecker_GC_malloc(int size, enum eMemCheckerKind obj_kind)
+{
+    void* obj = GC_MALLOC(size);
+    GC_REGISTER_FINALIZER(obj, memchecker_finalizer, NULL, NULL, NULL);
+
+    char buf[512];
+    snprintf(buf, 512, "%d", (unsigned int)obj);
+    hash_put(gMemChecker, buf, (void*)obj_kind);
+
+    return obj;
+}
+
+sRef* sRef_new(void* mem, enum eMemCheckerKind kind)
+{
+    sRef* self = memchecker_GC_malloc(sizeof(sRef), kMCRef);
+
+    self->mMem = mem;
+    self->mKind = kind;
+
+    return self;
+}
