@@ -836,16 +836,66 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
             memcpy(tmp2, arg0, tmp-arg0);
             tmp2[tmp - arg0] = 0;
 
-            if(object) {
+            if(strlen(tmp2) > 1 && tmp2[0] == '$') {
+                sRef* ref2 = NULL;
+                hash_obj* top_stack
+                     = vector_item(gStackFrameRefs, vector_size(gStackFrameRefs)-1);
+                if(hash_item(top_stack, tmp2 + 1)) {
+                    ref2 = hash_item(top_stack, tmp2 + 1);
+                }
+                else if(object && hash_item(object->mRefs, tmp2 + 1)) {
+                    ref2 = hash_item(object->mRefs, tmp2 + 1);
+                }
+                else {
+                    ref2 = hash_item(gRefs, tmp2 + 1);
+                }
+
+                if(ref2) {
+                    if(ref2->mKind == kMCObject) {
+                        object2 = ref2->mMem;
+                    }
+                    else {
+                        err_msg("invalid refference", sname, sline);
+                        return FALSE;
+                    }
+                }
+            }
+            else if(strlen(tmp2) > 2 && tmp2[0] == '0' && tmp2[1] == 'x') {
+                void* mem = (void*)(unsigned long)strtoll(tmp2, NULL, 16);
+
+                int kind = memchecker_is_enable_mem(mem);
+
+                if(kind == kMCObject) {
+                    object2 = mem;
+                }
+            }
+            else if(object) {
+                hash_obj* top_stack
+                     = vector_item(gStackFrameObjects, vector_size(gStackFrameObjects)-1);
+
                 if(strcmp(tmp2, "self") == 0) {
                     object2 = object;
                 }
-                else {
+                else if(hash_item(top_stack, tmp2)) {
+                    object2 = hash_item(top_stack, tmp2);
+                }
+                else if(object && hash_item(object->mObjects, tmp2)) { 
                     object2 = hash_item(object->mObjects, tmp2);
                 }
+                else {
+                    object2 = hash_item(gObjects, tmp2);
+                }
             }
             else {
-                object2 = hash_item(gObjects, tmp2);
+                hash_obj* top_stack
+                     = vector_item(gStackFrameObjects, vector_size(gStackFrameObjects)-1);
+
+                if(hash_item(top_stack, tmp2)) {
+                    object2 = hash_item(top_stack, tmp2);
+                }
+                else {
+                    object2 = hash_item(gObjects, tmp2);
+                }
             }
 
             if(object2 == NULL) {
@@ -877,26 +927,91 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
             klass = NULL;
         }
         else {
-            object2 = NULL;
-            method = NULL;
-            fun = hash_item(gFuncs, arg0);
-            inner_fun = hash_item(gInnerCommands, arg0);
-            if(object) {
-                klass = hash_item(object->mClasses, arg0);
-                if(klass == NULL) {
-                    sObject* parent = object->mParent;
-                    while(parent) {
-                        klass = hash_item(parent->mClasses, arg0);
-                        if(klass) break;
-                        parent = parent->mParent;
+            if(strlen(arg0) > 1 && arg0[0] == '$') {
+                sRef* ref2 = NULL;
+                hash_obj* top_stack
+                     = vector_item(gStackFrameRefs, vector_size(gStackFrameRefs)-1);
+                if(hash_item(top_stack, arg0 + 1)) {
+                    ref2 = hash_item(top_stack, arg0 + 1);
+                }
+                else if(object && hash_item(object->mRefs, arg0 + 1)) {
+                    ref2 = hash_item(object->mRefs, arg0 + 1);
+                }
+                else {
+                    ref2 = hash_item(gRefs, arg0 + 1);
+                }
+
+                if(ref2) {
+                    if(ref2->mKind == kMCFunction) {
+                        object2 = NULL;
+                        method = NULL;
+                        fun = ref2->mMem;
+                        inner_fun = NULL;
+                        klass = NULL;
+                    }
+                    else if(ref2->mKind == kMCClass) {
+                        object2 = NULL;
+                        method = NULL;
+                        fun = NULL;
+                        inner_fun = NULL;
+                        klass = ref2->mMem;
+                    }
+                    else {
+                        err_msg("invalid refference", sname, sline);
+                        return FALSE;
                     }
                 }
-                if(klass == NULL) {
-                    klass = hash_item(gClasses, arg0);
+                else {
+                    err_msg("invalid refference", sname, sline);
+                    return FALSE;
+                }
+            }
+            else if(strlen(arg0) > 2 && arg0[0] == '0' && arg0[1] == 'x') {
+                void* mem = (void*)(unsigned long)strtoll(arg0, NULL, 16);
+
+                int kind = memchecker_is_enable_mem(mem);
+
+                if(kind == kMCFunction) {
+                    object2 = NULL;
+                    method = NULL;
+                    fun = mem;
+                    inner_fun = NULL;
+                    klass = NULL;
+                }
+                else if(kind == kMCClass) {
+                    object2 = NULL;
+                    method = NULL;
+                    fun = NULL;
+                    inner_fun = NULL;
+                    klass = mem;
+                }
+                else {
+                    err_msg("invalid address", sname, sline);
+                    return FALSE;
                 }
             }
             else {
-                klass = hash_item(gClasses, arg0);
+                object2 = NULL;
+                method = NULL;
+                fun = hash_item(gFuncs, arg0);
+                inner_fun = hash_item(gInnerCommands, arg0);
+                if(object) {
+                    klass = hash_item(object->mClasses, arg0);
+                    if(klass == NULL) {
+                        sObject* parent = object->mParent;
+                        while(parent) {
+                            klass = hash_item(parent->mClasses, arg0);
+                            if(klass) break;
+                            parent = parent->mParent;
+                        }
+                    }
+                    if(klass == NULL) {
+                        klass = hash_item(gClasses, arg0);
+                    }
+                }
+                else {
+                    klass = hash_item(gClasses, arg0);
+                }
             }
         }
 
@@ -909,25 +1024,48 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
             klass->input = input;
 
             if(vector_size(argv) >= 2) {
+                BOOL global = FALSE;
+                BOOL local = FALSE;
+                int i;
+                for(i=1; i<vector_size(argv); i++) {
+                    char* arg = string_c_str(vector_item(argv, i));
+                    if(strcmp(arg, "-l") == 0) {
+                        vector_erase(argv, i);
+                        i--;
+                        continue;
+                    }
+                    else if(strcmp(arg, "-g") == 0) {
+                        vector_erase(argv, i);
+                        i--;
+                        continue;
+                    }
+                }
                 char* objname = string_c_str(vector_item(argv, 1));
 
+                sObject* object2 = sObject_new(objname, klass, object);
+                if(local) {
+                    hash_obj* top_stack
+                         = vector_item(gStackFrameObjects, vector_size(gStackFrameObjects)-1);
+
+                    hash_put(top_stack, objname, object2);
+                }
+                else if(object && !global) {
+                    hash_put(object->mObjects, objname, object2);
+                }
+                else {
+                    hash_put(gObjects, objname, object2);
+                }
+
                 /// スタックフレームの記録を付ける
                 vector_add(gStackTraceFName, STRING_NEW(sname));
                 vector_add(gStackTraceLineNum, (void*)sline);
 
                 // スタックを掘る
                 vector_add(gStackFrame, HASH_NEW(30));
-                vector_add(gStackFramePointers, HASH_NEW(30));
                 vector_add(gStackFrameHashs, HASH_NEW(30));
                 vector_add(gStackFrameArrays, HASH_NEW(30));
-
-                sObject* object2 = sObject_new(objname, klass, object);
-                if(object) {
-                    hash_put(object->mObjects, objname, object2);
-                }
-                else {
-                    hash_put(gObjects, objname, object2);
-                }
+                vector_add(gStackFrameObjects, HASH_NEW(30));
+                vector_add(gStackFrameRefs, HASH_NEW(30));
 
                 *rcode = run(klass->statments, title, nextout
                                 , nextin, nexterr
@@ -938,8 +1076,9 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 // スタックを埋める
                 vector_pop_back(gStackFrame);
                 vector_pop_back(gStackFrameArrays);
+                vector_pop_back(gStackFrameObjects);
                 vector_pop_back(gStackFrameHashs);
-                vector_pop_back(gStackFramePointers);
+                vector_pop_back(gStackFrameRefs);
 
                 /// スタックフレームの記録を消す
                 string_obj* sname_strace = vector_pop_back(gStackTraceFName);
@@ -982,9 +1121,10 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
 
                 // スタックを掘る
                 vector_add(gStackFrame, HASH_NEW(30));
-                vector_add(gStackFramePointers, HASH_NEW(30));
                 vector_add(gStackFrameHashs, HASH_NEW(30));
                 vector_add(gStackFrameArrays, HASH_NEW(30));
+                vector_add(gStackFrameObjects, HASH_NEW(30));
+                vector_add(gStackFrameRefs, HASH_NEW(30));
 
                 if(object) {
                     klass = hash_item(object->mClasses, klassname);
@@ -1021,7 +1161,8 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 vector_pop_back(gStackFrame);
                 vector_pop_back(gStackFrameArrays);
                 vector_pop_back(gStackFrameHashs);
-                vector_pop_back(gStackFramePointers);
+                vector_pop_back(gStackFrameObjects);
+                vector_pop_back(gStackFrameRefs);
 
                 /// スタックフレームの記録を消す
                 string_obj* sname_strace = vector_pop_back(gStackTraceFName);
@@ -1055,9 +1196,10 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
 
                 // スタックを掘る
                 vector_add(gStackFrame, HASH_NEW(30));
-                vector_add(gStackFramePointers, HASH_NEW(30));
                 vector_add(gStackFrameHashs, HASH_NEW(30));
                 vector_add(gStackFrameArrays, HASH_NEW(30));
+                vector_add(gStackFrameObjects, HASH_NEW(30));
+                vector_add(gStackFrameRefs, HASH_NEW(30));
 
                 sBlock* block = vector_item(blocks, 0);
                 sStatments* statments = block->mStatments;
@@ -1071,8 +1213,9 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 // スタックを埋める
                 vector_pop_back(gStackFrame);
                 vector_pop_back(gStackFrameArrays);
+                vector_pop_back(gStackFrameObjects);
                 vector_pop_back(gStackFrameHashs);
-                vector_pop_back(gStackFramePointers);
+                vector_pop_back(gStackFrameRefs);
 
                 /// スタックフレームの記録を消す
                 string_obj* sname_strace = vector_pop_back(gStackTraceFName);
@@ -1134,9 +1277,10 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 hash_put(object2->mArrays, "ARGV", fun_argv);
 
                 vector_add(gStackFrame, HASH_NEW(30));
-                vector_add(gStackFramePointers, HASH_NEW(30));
                 vector_add(gStackFrameHashs, HASH_NEW(30));
                 vector_add(gStackFrameArrays, HASH_NEW(30));
+                vector_add(gStackFrameObjects, HASH_NEW(30));
+                vector_add(gStackFrameRefs, HASH_NEW(30));
             }
             /// 引数名が指定されていなければスタックフレームは増やさない
             else {
@@ -1182,8 +1326,9 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 /// スタックを消す ///
                 vector_pop_back(gStackFrame);
                 vector_pop_back(gStackFrameArrays);
+                vector_pop_back(gStackFrameObjects);
                 vector_pop_back(gStackFrameHashs);
-                vector_pop_back(gStackFramePointers);
+                vector_pop_back(gStackFrameRefs);
 
                 /// 引数を消す ///
                 hash_erase(object2->mArrays, "ARGV");
@@ -1247,7 +1392,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
 
                 /// 引数を渡す ///
                 int k;
-                fun_argv = VECTOR_NEW(30);
+                fun_argv = sAry_new(30);
                 for(k=1; k<vector_size(argv); k++) {
                     char* value = string_c_str(vector_item(argv, k));
                     vector_add(fun_argv, STRING_NEW(value));
@@ -1256,9 +1401,10 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 hash_put(gArrays, "ARGV", fun_argv);
 
                 vector_add(gStackFrame, HASH_NEW(30));
-                vector_add(gStackFramePointers, HASH_NEW(30));
                 vector_add(gStackFrameHashs, HASH_NEW(30));
                 vector_add(gStackFrameArrays, HASH_NEW(30));
+                vector_add(gStackFrameObjects, HASH_NEW(30));
+                vector_add(gStackFrameRefs, HASH_NEW(30));
             }
             /// 引数名が指定されていなければスタックフレームは増やさない
             else {
@@ -1268,7 +1414,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
 
                 /// 引数を渡す ///
                 int k;
-                fun_argv = VECTOR_NEW(30);
+                fun_argv = sAry_new(30);
                 for(k=1; k<vector_size(argv); k++) {
                     char* value = string_c_str(vector_item(argv, k));
                     vector_add(fun_argv, STRING_NEW(value));
@@ -1303,8 +1449,9 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
                 /// スタックを消す ///
                 vector_pop_back(gStackFrame);
                 vector_pop_back(gStackFrameArrays);
+                vector_pop_back(gStackFrameObjects);
                 vector_pop_back(gStackFrameHashs);
-                vector_pop_back(gStackFramePointers);
+                vector_pop_back(gStackFrameRefs);
 
                 /// 引数を消す ///
                 hash_erase(gArrays, "ARGV");
